/*
 *    NzdlCacheWrapper.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//the package we're in
package org.nzdl.gsdl.service;

import java.util.Hashtable;

/* gsdl corba stuff */
import org.nzdl.gsdl.corba.gsdlInterface.corbaFilterRequest;
import org.nzdl.gsdl.util.NzdlCorbaFactory;

/**
 *  A Wrapper around an NzdlService object that adds functionality. 
 *
 * Typically one or more of the methods will be overridden to 
 * add that functionality
 *
 * @author Brett Sheeran (bsheeran@xtra.co.nz) (comments)
 * @author Stuart Yeates  (say1@cs.waikato.ac.nz)
 * @version $Revision: 2688 $
 */
public class NzdlCacheWrapper
  extends NzdlWrapper implements java.io.Serializable{

  private Hashtable _docs;
  private Hashtable _collectionInfo;
  private Hashtable _services;

  /**
   * The bean constructor ...
   */  
  public NzdlCacheWrapper() {
    _docs = new Hashtable();
    _collectionInfo = new Hashtable();
    _services = new Hashtable();
  }

  /**
   * The normal constructor ...
   */  
  public NzdlCacheWrapper(NzdlService service) {
    super(service);
    _docs = new Hashtable();
    _collectionInfo = new Hashtable();
    _services = new Hashtable();
  }
  
  /**
   * Wraps a getCollectionInfo with a cache check 
   * to see if we've made the same call before ...
   * @see NzdlService#getCollectionInfo
   */
  public NzdlCollectionInfo getCollectionInfo( String _name ) {
    //System.err.println("in NzdlServiceCachingClient::getCollectionInfo()");
    NzdlCollectionInfo info = null;
    info = (NzdlCollectionInfo)_collectionInfo.get(_name);
    if (info != null)
      return info;
    info = super.getCollectionInfo(_name);
    _collectionInfo.put(_name,info);

    //System.err.println("Cache miss (getCollectionInfo)");
    return info;
  }

  /**
   * Wraps a getDocument with a cache check 
   * to see if we've made the same call before ...
   * @see NzdlService#getDocument
   */
  public String getDocument( String _name, String _docID ) {
    String doc = null;
    String key = "" + _name + _docID;
    doc = (String)_docs.get(key);
    if (doc != null)
      return doc;
    doc = super.getDocument(_name,_docID);
    _docs.put(key, doc);
    
    //System.err.println("Cache miss (getDocument) " + _name 
    //	        + " " +_docID);
    
    return doc;
  }

  /**
   * Wraps a service with a cache check 
   * to see if we've made the same call before ...
   * @see NzdlService#service
   */
  public void service( String _name, NzdlRequest _request, NzdlResponse _response ) {    
    
    String key = _name + corbaFilterRequestToString(_request);

    if (_services.get(key) != null) {
      _response.copyData((NzdlResponse) _services.get(key));
      return;
    }

    super.service(_name, _request,_response);

    _services.put(key, _response);
    
    //System.err.println("Cache miss (service)" + key);
    return;
  }
  
  /**
   * Stringifies a NzdlRequest object so we can use it as a hash value
   * @see NzdlRequest
   */
  protected String corbaFilterRequestToString(NzdlRequest nzdlrequest)
  {
    corbaFilterRequest request = nzdlrequest.getFilter();
    StringBuffer buf = new StringBuffer(1000);
    
    String tmp = NzdlCorbaFactory.toString(request.filter);
    buf.append(tmp).append('\uFFFF');
    
    for (int i = 0; i <request.filterOptions.length;i++){
      tmp = NzdlCorbaFactory.toString(request.filterOptions[i].name);
      buf.append(tmp).append('\uFFFF');
      tmp = NzdlCorbaFactory.toString(request.filterOptions[i].value);
      buf.append(tmp).append('\uFFFF');
    }
    
    for (int i = 0; i <request.docSet.length;i++){
      tmp = NzdlCorbaFactory.toString(request.docSet[i]);
      buf.append(tmp).append('\uFFFF');
    }
    
    buf.append(request.filterResultOptions).append('\uFFFF');

    tmp = NzdlCorbaFactory.toString(request.requestParams);
    buf.append(tmp).append('\uFFFF');
    
    tmp = NzdlCorbaFactory.toString(request.refParams);
    buf.append(tmp).append('\uFFFF');
    
    for (int i = 0; i <request.fields.length;i++){
      tmp = NzdlCorbaFactory.toString(request.fields[i]);
      buf.append(tmp).append('\uFFFF');
    }
    
    buf.append(request.getParents).append('\uFFFF');

    return buf.toString();
  }

}
